#############################################################################
#
# Computes different kinds of ideal shares
# date:     2009/03/26
# modified: 2009/03/26
# 
#############################################################################

idealshare <-
	function(votes, housesize=NULL, districtMagnitudes=NULL, partySeats=NULL,
		acc=1e-10) {

	# Checks
	if (is.null(housesize) && is.null(districtMagnitudes) &&
		is.null(partySeats))
			stop("Specify at least one basis for ideal share.")
	if (is.matrix(votes)) {
		if (!is.null(housesize) && (!is.null(districtMagnitudes) ||
			!is.null(partySeats))) {
				warning(paste("Housesize is provided and district",
					"magnitudes and/or party seats, removing housesize"))
				housesize <- NULL
		}
		if (!is.null(districtMagnitudes) && !is.null(partySeats)) {
			if (sum(districtMagnitudes) != sum(partySeats))
				stop(paste("Sum of district magnitudes differs from the",
					"sum of party votes"))
		}
		if (!is.null(districtMagnitudes)) 
			if (nrow(votes) != length(districtMagnitudes))
				stop("District magnitudes do not fit vote matrix")
		if (!is.null(partySeats))
			if (ncol(votes) != length(partySeats))
				stop("Party seats do not fit vote matrix")

		if (is.null(colnames(votes)))
			colnames(votes) <- paste("P", seq(1, ncol(votes)), sep="")
	} else if (is.vector(votes)) {
		if (is.null(housesize)) {
			stop("House size is required for vector problems")
		}
		if (!(is.null(districtMagnitudes) && is.null(partySeats))) {
			warning(paste("District magnitudes and/or party seats are",
				"provided for vector problems, ignoring them."))
			districtMagnitudes <- NULL
			partySeats <- NULL
		}
	} else
		stop("Votes have a wrong class")

	# Ideal share computation
	if (is.matrix(votes)) {
		if (!is.null(housesize)) {
			quota <- sum(votes)/housesize
			idealshare <- votes/quota
			isMethod <- "overall"
		} else if (!is.null(districtMagnitudes)) {
			if (is.null(partySeats)) {
				quota <- rowSums(votes)/districtMagnitudes
				idealshare <- 
					diag(1/quota)%*%votes
				isMethod <- "district"
			} else {
				newIPF <- .jnew("rjBazi/BipropApp", 
					as.double(as.vector(t(votes))), 
					as.integer(districtMagnitudes),
					as.integer(partySeats),
					colnames(votes))
				.jcall(newIPF, "V", "calculateIdealshare", acc)
				rowDiv <- .jcall(newIPF, "[D", "getRowDiv")
				colDiv <- .jcall(newIPF, "[D", "getColDiv")
				idealshare <- matrix(.jcall(newIPF, "[D", "getScaledWeights"),
					nrow=nrow(votes), byrow=TRUE)
				error <- .jcall(newIPF, "D", "getError")
				iterations <- .jcall(newIPF, "[I", "getAlgoPerformanceMeasures")
				names(iterations) <- .jcall(newIPF, "[S", "getAlgoPerformMeasureNames")
				isMethod <- "matrix"
			}
		} else {
			quota <- colSums(votes)/partySeats
			idealshare <- 
				votes%*%diag(1/quota)
			isMethod <- "party"
		}
		rownames(idealshare) <- rownames(votes)
		colnames(idealshare) <- colnames(votes)
					
	} else {
		idealshare <- votes/sum(votes) * housesize
		isMethod <- "vector"
		names(idealshare) <- names(votes)
	}

	# collect information
	retObj <- list()
	retObj$idealshare <- idealshare
	retObj$isMethod <- isMethod
	if (isMethod == "matrix") {
		retObj$rowDiv <- rowDiv
		retObj$colDiv <- colDiv
		retObj$iterations <- iterations
		retObj$error <- error
	} else {
		retObj$quota <- quota
	}

	return(retObj)
}
