##############################################################################
#
# date:     2007/02/20
# modified: 2009/05/03
#
#
##############################################################################

write.baziData <- function(file, baziData) {

	argList <- list()
	argList[["file"]] <- file
	if (is.element("districtMagnitudes", names(baziData))) {
		districtMethod <- ifelse(baziData$separate, "separate",
			ifelse(baziData$nzz, "nzz", "biprop"))
	} else {
		districtMethod <- NULL
	}
	for (nn in names(baziData)) {
		if (nn == "separate" || nn == "nzz" || nn == "pq")
			next
		if (identical(baziData[[nn]], NA)) {
			argList[[nn]] <- NULL
		} else {
			argList[[nn]] <- baziData[[nn]]
		}	
	}
	if (!is.null(districtMethod)) {
		argList[["districtMethod"]] <- districtMethod
	}
	
	# compatibility
	names(argList) <- sub("colM", "minConstrSuper", names(argList))
	
	do.call("write.bazi", argList)

}

write.bazi <- function(file, seats=NULL, votes, appMethod, districtMethod=NULL,
                        minConstr=NULL, minConstrSuper=NULL, maxConstr=NULL,
                        directSeats=NULL, naiv=NULL, listCombinations=NULL,
                        info=NULL, columnNames=c("Party", "Votes"),
                        name=sub(".bazi", "", file), 
                        houseSize=NULL, districtMagnitudes=NULL) {

	if (!is.null(seats)) {
	
		warning(paste("Argument 'seats' will be not available in future",
			" versions, please use 'houseSize' for vector problems or",
			" 'districtMagnitudes' for matrix problems instead.", sep=""))
	}

	# some initial checks
	if (!(is.null(minConstr) || is.null(minConstrSuper))) 
	stop("Options 'minConstr' and 'minConstrSuper' must not be combined.")
	if (!is.null(directSeats) && !(is.null(minConstr) && is.null(maxConstr)))
	    stop(paste("Direct seats cannot be combined with minimum or maximum",
		"constraints", sep=""))
	if (length(columnNames) != 2) stop("Column names must have 2 elements.")
	
	if (!(is.null(houseSize) || is.null(districtMagnitudes))) {
		stop("Options 'houseSize' and 'districtMagnitudes' are exclusive")
	}
	
	if (!is.null(seats) && !(is.null(houseSize) && is.null(districtMagnitudes))) {
		stop(paste("Argument 'seats' is incompatible with options ",
			"'houseSize' and 'districtMagnitudes'.", sep=""))
	}
	
	if (is.null(seats) && is.null(houseSize) && is.null(districtMagnitudes)) {
		stop("Please specify either 'houseSize' or 'districtMagnitudes'")
	}
	# for historical reasons
	if (!is.null(houseSize)) {
		seats <- houseSize
	} else if (!is.null(districtMagnitudes)) {
		seats <- districtMagnitudes
	}
	
	# Header for Bazifile
	startBazidata <- paste("=TITEL= ", name, "\n=METHODE= ", 
	    paste(appMethod, sep="", collapse=", "), "\n=AUSGABE= verhor, ",
		"Div/Quote, kodiert", sep="")
	

	if (is.matrix(votes)) {	# biproporitonal data

	    # matrix checks
            if (is.null(districtMethod) || !(districtMethod == "separate" ||
		districtMethod == "nzz" || districtMethod == "biprop")) 
		    stop(paste("Specified district method missing wrong, use ",
			"'separate', 'nzz', or 'biprop'.", sep=""))
	    if (districtMethod == "separate" && !is.null(minConstrSuper)) 
		stop(paste("No global minimum requirements for separate district",
                    " evaluation.", sep=""))
	    if (districtMethod != "separate" && !is.null(maxConstr)) 
		warning(paste("No maximum requirements for biproportional ",
		    "apportionments, ignoring maximum requirements.", sep=""))
            if (districtMethod != "separate" && !is.null(directSeats)) 
		warning(paste("No direct seats for biproportional ",
                    "apportionments, ignoring direct seats.", sep=""))
	    if ((districtMethod == "biprop" || districtMethod == "nzz") &&
	    	!is.null(listCombinations))
                    stop("Biprop methods do not allow list combinations.")
		
	    inputData <- paste("=EINGABE= ",  columnNames[1],",", 
		columnNames[2], sep = "")
	    if (districtMethod == "separate") {
		inputData <- paste(inputData, 
                        ifelse(!is.null(directSeats), "Direktmandate",
                        ifelse(!is.null(minConstr)&&!is.null(maxConstr), "MinToMax", 
                        ifelse(!is.null(minConstr), "Minimum",
                        ifelse(!is.null(maxConstr), "Maximum", "---"
		    )))), sep=",")
	    } else
		inputData <- paste(inputData, 
                    ifelse(is.null(minConstr), 
		    ifelse(is.null(minConstrSuper), "---", "MIN(global)"
                    ), "min(lokal)" ), sep=",")
		
	    output <- paste(startBazidata, inputData, sep="\n")
            # checks
	    dimV <- dim(votes)
	    if (dimV[2] == 1) {
		votes <- t(votes)
		dimV <- dim(votes)
	    }
            rows <- dimV[1]
	    cols <- dimV[2]

	    if (length(seats) != dimV[1]) 
		stop("District magnitudes to not fit to vote matrix.")
	    if (!is.null(minConstr) && !all(dim(minConstr) == dimV)) 
		stop("Minimum requirements do not fit to vote matrix.")
	    if (!is.null(minConstrSuper) && length(minConstrSuper) != dimV[2]) 
		stop(paste("Minimum requirements for superapportionment do not",
		    " fit to vote matrix.", sep=""))
	    if (!is.null(listCombinations) && !all(dim(listCombinations) == dimV))
		stop("List combinations do not fit to vote matrix.")
		
	    # row/col names
	    if (is.null(rownames(votes))) 
		rownames(votes) <- paste("D", seq(1, rows), sep="")
	    if (is.null(colnames(votes))) 
		colnames(votes) <- paste("P", seq(1, cols), sep="")	
		
	    districtNames <- rownames(votes)
	    partyNames <- colnames(votes)
	    if (any(grep(" ", partyNames) > 0)) 
		partyNames <- paste("\"", partyNames, "\"", sep="")
		
	    # district option
	    output <- paste(output, paste("=DISTRIKTOPTION= ", districtMethod,
		sep=""), sep="\n")

	    # it's not clear to me anymore why the stuff is transposed, anyway
	    # 2009/03/25: maybe within districts parties in cols
	    votes <- t(votes)
	    if (!is.null(minConstr)) minConstr <- t(minConstr)
	    if (!is.null(directSeats)) directSeats <- t(directSeats)
	    if (!is.null(listCombinations)) listCombinations <- t(listCombinations)

	    # create output string
	    for (i in 1:dimV[1]) {
		districtHeader <- paste("=DISTRICT= ", districtNames[i], 
		    "\n=MANDATE= ", seats[i], "\n=DATEN= ", sep="")
		listCombIndicator <- rep("", dimV[2])

		# take care on list combinations
		if (!is.null(listCombinations)) {
		    tmpOrder <- order(listCombinations[,i])
		    tmpListComb <- listCombinations[tmpOrder,i]
		    for (j in 2:dimV[2]) {
			if (tmpListComb[j] > 0 && 
			    tmpListComb[j] == tmpListComb[j-1]) 
			    listCombIndicator[j] <- "+"
		    }	
		} else {
		    tmpOrder <- seq(1,dimV[2])
		}
		if (i == 1 && !is.null(minConstrSuper)) 
                    tmpOut <- paste( 
			listCombIndicator, partyNames[tmpOrder],
                            votes[tmpOrder,i], minConstrSuper[tmpOrder],
                                sep="\t")
		else if (!is.null(minConstr)) 
		    tmpOut <- paste(
			listCombIndicator, partyNames[tmpOrder], 
			    votes[tmpOrder,i], minConstr[tmpOrder,i], sep="\t")
		else if (!is.null(directSeats)) 
		    tmpOut <- paste( 
			listCombIndicator, partyNames[tmpOrder], 
			    votes[tmpOrder,i], directSeats[tmpOrder,i],
                                sep="\t")
		else tmpOut <- paste( 
			listCombIndicator, partyNames[tmpOrder],
                            votes[tmpOrder,i], rep("null", dimV[2]), sep="\t")
		# append output
		output <- paste(output, districtHeader,
		    paste(tmpOut, collapse="\n"), sep="\n")
	    }	
	}
	else {	# vector apportionment
		
	    if (!is.null(minConstrSuper)) {
		wString <- paste("Minimum constraints for superapportionment are",
		    "not useful for vector apportionment.")
		if (is.null(minConstr)) {
		    minConstr <- minConstrSuper
		    warning(paste(wString, "Using them as minimum requiremtens."))
		} else {
                    minConstrSuper <- NULL
		    warning(paste(wString, "Not using them."))
		}
	    }	
		
	    inputData <- paste("=EINGABE= ", columnNames[1], ",", columnNames[2],
	    	",", ifelse(!is.null(directSeats), "Direktmandate",
			ifelse(!is.null(minConstr) && !is.null(maxConstr), 
				"MinToMax", 
			    ifelse(!is.null(minConstr), "Minimum",
				ifelse(!is.null(maxConstr), "Maximum",
				    ifelse(!is.null(naiv), "Etc", "---"
		    ))))), sep="")
	    output <- paste(startBazidata, inputData, sep="\n")
		
	    houseHeader <- paste("=MANDATE= ", paste(seats, sep="",
                            collapse=", "), "\n=DATEN=", sep="")
	    noOfParties <- length(votes)
            if (!is.null(minConstr) && noOfParties!= length(minConstr)) 
		stop("Minimum requirements do not fit to vote vector.")
	    if (!is.null(maxConstr) && noOfParties!= length(maxConstr)) 
		stop("Maximum requirements do not fit to vote vector.")
	    if (!is.null(directSeats) && noOfParties != length(directSeats)) 
		stop("Direct seats do not fit to vote vector.")
	    if (!is.null(naiv) && noOfParties != length(naiv)) 
		stop("Naiv apportionment does not fit to vote vector.")
	    if (!is.null(listCombinations) &&
                noOfParties != length(listCombinations))
		    stop("List combinations do not fit votes.")
	    if (is.null(names(votes))) 
		names(votes) <-paste("P" ,seq(1, noOfParties), sep="")
		
	    partyNames <- names(votes)
            if (any(grep(" ", partyNames) > 0)) 
		partyNames <- paste("\"", names(votes), "\"", sep="")
		
	    if (!is.null(listCombinations)) {
		tmpOrder <- order(listCombinations)
		listCombinations <- listCombinations[tmpOrder]
		listCombIndicator <- rep("", noOfParties)
		for (j in 2:noOfParties) {
		    if (listCombinations[j] > 0 && 
			    listCombinations[j] == listCombinations[j-1]) 
				listCombIndicator[j] <- "+"
		}	
		partyNames <- mapply(paste, listCombIndicator, 
		    partyNames[tmpOrder], sep="")
		votes <- votes[tmpOrder]
		if (!is.null(minConstr)) minConstr <- minConstr[tmpOrder]
		if (!is.null(maxConstr)) maxConstr <- maxConstr[tmpOrder]
		if (!is.null(directSeats)) directSeats <- directSeats[tmpOrder]
		if (!is.null(naiv)) naiv <- naiv[tmpOrder]
	    }	
			
		
	    tmpOut <- paste(partyNames, votes, sep="\t")
		
	    if (!is.null(minConstr) && !is.null(maxConstr)) 
		tmpOut <- paste(tmpOut, paste(minConstr, maxConstr, sep=".."),
		    sep="\t")
	    else if (!is.null(minConstr)) 
		tmpOut <- paste(tmpOut, minConstr, sep="\t")
	    else if (!is.null(maxConstr)) 
		tmpOut <- paste(tmpOut, maxConstr, sep="\t")
	    else if (!is.null(directSeats)) 
		tmpOut <- paste(tmpOut, directSeats, sep="\t")
	    else if (!is.null(naiv)) 
		tmpOut <- paste(tmpOut, naiv, sep="\t")
	    else 
		tmpOut <- paste(tmpOut, rep("null", noOfParties), sep="\t")
		
	    tmpOut <- paste(tmpOut, sep="", collapse="\n")
	    output <- paste(output, houseHeader, tmpOut, sep="\n")
	
	}
	if (is.null(info)) output <- paste(output, "=ENDE=", sep="\n")
	else output <- paste(output, "=INFO=", info, "=ENDE=", sep="\n")

	cat(output, file=file)
}
